<?php

namespace Galaxy\NacosClient\Nacos;

class Nacos
{

    public static  $configs = [];
    public static $basePath = '';
    public static function run($options)
    {
        $res = self::configure($options);
        if (empty($res)){
            return;
        }
        self::info('开始拉取配置文件');
        $url =  self::$configs['NACOS_URL'] ?? '';
        $group = self::$configs['GROUP'] ?? '';
        if (empty($url)){
            self::info('无nacos链接');
            return;
        }
        $modelPath = self::$basePath.'.env';
        $envFile = file_get_contents($modelPath);
        $configGroup = 'APP_NAME='.$group;
        if (!preg_match("/$configGroup/", $envFile)) {
            self::info($group . '应用不匹配');
            return;
        }

        $key = $group . '-env-' . self::$configs['APP_ENV'];
        $res = self::getConfig($key, $group);
        if (empty($res)) {
            $log = $key . '配置不存在';
            self::info($log);
            return;
        }
        try {
            $cacheKey = 'fileMd5' . $key;
            $md5File = md5($res);
            $oldMd5 = Nacos::NacosGet($cacheKey);
            if (!$oldMd5) {
                self::write($modelPath, $res);
                Nacos::NacosPut($cacheKey, $md5File);
                $logs = '首次获取配置' . $key;
                self::info($logs);
                return;
            }
            $logs = $key . '文件未发生变更' . PHP_EOL;
            if ($oldMd5 != $md5File) {
                $logs = $key . '文件发生变更,修改配置项' . PHP_EOL;
                Nacos::NacosPut($cacheKey, $md5File);
                self::write($modelPath, $res);
            }
            self::info($logs);
        } catch (\Exception $e) {
            self::info(['msg'=>'获取配置失败'. $key.':'.$e->getMessage(),'ex'=> $e]);
        }
    }

    /**
     * 写入配置
     * @param $path
     * @param $res
     * @return void
     * @throws \Exception
     */
    public static function write($path,$res){
        $save = file_put_contents($path, $res);
        if ($save === false){
            throw new \Exception('保存配置失败'.$path);
        }
    }

    /**
     * 获取nacos配置
     * @param $dataId
     * @param $group
     * @return false|mixed|string
     * @throws \Exception
     */
    public static function getConfig($dataId, $group = 'DEFAULT_GROUP')
    {
        try {
            return self::requery($dataId, $group);
        } catch (\Exception $e) {
            $code = $e->getCode();
            if ($code == 403) {
                self::getToken(true);
                return self::requery($dataId, $group);
            }
            return false;
        }
    }

    /**
     * 发起请求
     * @param $dataId
     * @param $group
     * @return mixed|string
     * @throws \Exception
     */
    public static function requery($dataId, $group)
    {
        $url = self::$configs['NACOS_URL'].'/nacos/v2/cs/config?accessToken='.self::getToken().'&dataId='.$dataId.'&group='.$group;
        return self::getConfigRequest($url);
    }

    /**
     * 获取token
     * @param $new
     * @return mixed|string|null
     */
    private static function getToken($new = false)
    {
        $key = 'nacos:sync:get:token';
        $token = Nacos::NacosGet($key);;
        if (empty($token) || $new) {
            $user = self::$configs['ACCOUNT'];
            $pass = self::$configs['NACOS_PWD'];
            $url  = self::$configs['NACOS_URL'].'/nacos/v1/auth/login';
            $data = ['username' => $user, 'password' =>$pass];
            $token = self::login($data ,$url);
            $logs = '获取token： ' . $token;
            self::info($logs);
        }
        Nacos::NacosPut($key, $token);
        return $token;
    }

    /**
     * 请求nacos
     * @param $url
     * @return mixed|string
     */
    public static function getConfigRequest($url){
        // 创建一个cURL资源
        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "GET",
            CURLOPT_HTTPHEADER => array(
                "Content-Type: application/json",
                "Access-Type: application/json",
                "cache-control: no-cache"
            ),
        ));
        $response = curl_exec($curl);
        $err = curl_error($curl);
        if ($err){
            self::info($err);
        }
        curl_close($curl);
        $data = json_decode($response,true);
        if (isset($data['status']) && $data['status'] == 403){
            throw new \Exception('token过期',403);
        }
        if (isset($data['code']) && $data['code'] !== 0){
            self::info('获取配置返回结果：'.$response);
            return '';
        }
        return $data['data'] ?? '';
    }


    /**
     * 登入
     * @param $data
     * @param $url
     * @return mixed|string|void
     */
    private static function login($data,$url)
    {
        try {
            $curl = curl_init();
            curl_setopt_array($curl, array(
                CURLOPT_URL => $url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 30,
                CURLOPT_POSTFIELDS => http_build_query($data),
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => "POST",
                CURLOPT_HTTPHEADER => array(
                    "Content-Type: application/x-www-form-urlencoded",
                    "cache-control: no-cache"
                ),
            ));
            $response = curl_exec($curl);
            $err = curl_error($curl);
            if ($err){
                self::info($err);
            }
            curl_close($curl);
            // 处理响应
            $loginDataArr = json_decode($response,true);
            return $loginDataArr['accessToken'] ?? '';
        } catch (\Exception $e) {
            self::info($e->getMessage());
        }
    }

    /**
     * 读文件
     * @param $key
     * @return mixed|string
     */
    public static function NacosGet($key){
        $path =self::getPath(sys_get_temp_dir(), '/nacos.cache-'.md5(self::$configs['GROUP']));
        $data = json_decode(
            file_get_contents($path),
            true,
        );
        if (empty($data)){
            return '';
        }
        return $data[$key] ?? '';
    }

    /**
     * 写入文件
     * @param $key
     * @param $value
     * @return void
     */
    public static function NacosPut($key,$value){
        $path = self::getPath(sys_get_temp_dir(), '/nacos.cache-'.md5(self::$configs['GROUP']));
        $data = json_decode(
            file_get_contents($path),
            true,
        );
        $data[$key] = $value;
        file_put_contents(
            $path,
            json_encode($data),
        );
    }

    /**
     * 写日志
     * @param $msg
     * @return void
     */
    private static function info($msg)
    {
        self::writeLog($msg);
        if (!is_string($msg)){
            $msg = $msg['msg'];
        }
        echo $msg."\n";
    }

    /**
     * 获取文件路径
     * @param $fileName
     * @return string
     */
    public static function getPath($path,$fileName){

        if (!file_exists($path)) {
            mkdir($path, 0775, true);
            touch($fileName);
        }
        if (!file_exists($path.$fileName)) {
            touch($fileName);
        }
        return $path.$fileName;

    }

    /**
     * 获取nacos
     * @return bool
     */
    public static function configure($options)
    {
        $envNacos       = $options['env.nacos'];
        self::$basePath = $options['base_path'];
        $file = self::$basePath.$envNacos;
        $data = parse_ini_file($file);
        if (empty($data)) {
            self::info( '输入参数有误:'.$file);
            return false;
        }
        Nacos::$configs = $data;
        return true;
    }

    /**
     * 写入日志
     * @return void
     */
    public static function writeLog($logData){
        $logFile = '/nacos-'.date('Y-m-d').'.log';
        $path = self::getPath(sys_get_temp_dir(),$logFile);
        $data = json_decode(
            file_get_contents($path),
            true,
        );
        if (!is_string($logData)){
            $logData = json_encode($logData,);
        }
        $data[] = $logData;
        file_put_contents(
            $path,
            json_encode($data),
        );
    }
}